"use strict";
/**
 * Copyright 2021 Google LLC.
 * Copyright (c) Microsoft Corporation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createClient = void 0;
const events_1 = require("events");
const browserProtocol = __importStar(require("devtools-protocol/json/browser_protocol.json"));
const jsProtocol = __importStar(require("devtools-protocol/json/js_protocol.json"));
const mergedProtocol = [...browserProtocol.domains, ...jsProtocol.domains];
// Generate classes for each Domain and store constructors here.
const domainConstructorMap = new Map();
// Base class for all domains.
class DomainImpl extends events_1.EventEmitter {
    _client;
    constructor(_client) {
        super();
        this._client = _client;
    }
}
for (let domainInfo of mergedProtocol) {
    // Dynamically create a subclass for this domain. Note: This class definition is scoped
    // to this for-loop, so there will be a unique ThisDomain definition for each domain.
    class ThisDomain extends DomainImpl {
        constructor(_client) {
            super(_client);
        }
    }
    // Add methods to our Domain for each available command.
    for (let command of domainInfo.commands) {
        Object.defineProperty(ThisDomain.prototype, command.name, {
            value: async function (params) {
                return await this._client.sendCommand(`${domainInfo.domain}.${command.name}`, params);
            },
        });
    }
    domainConstructorMap.set(domainInfo.domain, ThisDomain);
}
class CdpClientImpl extends events_1.EventEmitter {
    _cdpConnection;
    _sessionId;
    _domains;
    constructor(_cdpConnection, _sessionId) {
        super();
        this._cdpConnection = _cdpConnection;
        this._sessionId = _sessionId;
        this._domains = new Map();
        for (const [domainName, ctor] of domainConstructorMap.entries()) {
            this._domains.set(domainName, new ctor(this));
            Object.defineProperty(this, domainName, {
                get() {
                    return this._domains.get(domainName);
                },
            });
        }
    }
    /**
     * Returns command promise, which will be resolved wth the command result after receiving CDP result.
     * @param method Name of the CDP command to call.
     * @param params Parameters to pass to the CDP command.
     */
    sendCommand(method, params) {
        return this._cdpConnection.sendCommand(method, params, this._sessionId);
    }
    _onCdpEvent(method, params) {
        // Emit a generic "event" event from here that includes the method name. Useful as a catch-all.
        this.emit('event', method, params);
        // Next, get the correct domain instance and tell it to emit the strongly typed event.
        const [domainName, eventName] = method.split('.');
        const domain = this._domains.get(domainName);
        if (domain) {
            domain.emit(eventName, params);
        }
    }
}
/**
 * Creates a new CDP client object that communicates with the browser using a given
 * transport mechanism.
 * @param transport A transport object that will be used to send and receive raw CDP messages.
 * @returns A connected CDP client object.
 */
function createClient(cdpConnection, sessionId) {
    return new CdpClientImpl(cdpConnection, sessionId);
}
exports.createClient = createClient;
//# sourceMappingURL=cdpClient.js.map